package datastore

import (
	"reflect"
	"testing"

	"github.com/jmoiron/modl"
	"github.com/thermokarst/bactdb/models"
)

func insertUser(t *testing.T, tx *modl.Transaction) *models.User {
	// Test on a clean database
	tx.Exec(`DELETE FROM users;`)

	user := newUser()
	if err := tx.Insert(user); err != nil {
		t.Fatal(err)
	}
	return user
}

func newUser() *models.User {
	return &models.User{UserName: "Test User"}
}

func TestUsersStore_Get_db(t *testing.T) {
	tx, _ := DB.Begin()
	defer tx.Rollback()

	want := insertUser(t, tx)

	d := NewDatastore(tx)

	user, err := d.Users.Get(want.Id)
	if err != nil {
		t.Fatal(err)
	}

	normalizeTime(&want.CreatedAt, &want.UpdatedAt, &want.DeletedAt)
	if !reflect.DeepEqual(user, want) {
		t.Errorf("got user %+v, want %+v", user, want)
	}
}

func TestUsersStore_Create_db(t *testing.T) {
	tx, _ := DB.Begin()
	defer tx.Rollback()

	user := newUser()

	d := NewDatastore(tx)

	created, err := d.Users.Create(user)
	if err != nil {
		t.Fatal(err)
	}

	if !created {
		t.Error("!created")
	}
	if user.Id == 0 {
		t.Error("want nonzero user.Id after submitting")
	}
}

func TestUsersStore_List_db(t *testing.T) {
	tx, _ := DB.Begin()
	defer tx.Rollback()

	user := insertUser(t, tx)
	want := []*models.User{user}

	d := NewDatastore(tx)

	users, err := d.Users.List(&models.UserListOptions{ListOptions: models.ListOptions{Page: 1, PerPage: 10}})
	if err != nil {
		t.Fatal(err)
	}

	for _, u := range want {
		normalizeTime(&u.CreatedAt, &u.UpdatedAt, &u.DeletedAt)
	}
	if !reflect.DeepEqual(users, want) {
		t.Errorf("got users %+v, want %+v", users, want)
	}
}
